/*********************************************
 * OPL 12.6.2.0 Model
 * Author: aguinet
 * Creation Date: 20 avr. 2017 at 09:14:32
 *********************************************/

/* Paramtres */
int Tper = ...;
int Mact = ...;
int Nbpat = ...;
int Nuni =...;
int TGV =...;

range P = 1..Tper;
range M = 0..Tper;
range J = 1..Mact;
range L = 2..Mact;
range N = 1..Nbpat;

int Dur[N][J] =...;
int Dtot[N] =...;
int Dtard[N] =...;
int Pred[J][J] =...;
float Cap[J][P] =...;

/* Variables de dcision */
dvar boolean Y[N][P][J];
dvar float+ X[J][M];
dvar float+ Cre[J][M];
dvar float+ Z[N][P];
dvar float+ Tach[N][J];
dvar float+ Dfin[N];
dvar float+ Rmax[J];
dvar float+ Cmax;
dvar float+ Dmoy;
dvar float+ Dmax;
dvar float+ Dmin;

/* Objectif : Admettre les patients au plus tt */

/* Minimiser Z2 l'emploi maximum des ressources */
/* minimize sum(j in J) Rmax[j]; */
/* Minimiser Z1 le temps d'attente des patients  admettre en HAD */
minimize sum(i in N : Dtard[i] > 0) (Dfin[i] - Dtot[i]);

/* Avec les contraintes */
subject to
{

  /* Approche Bi-objectif Lexicographique : intgration de l'objectif d'admission des patients au plus tt, Z1 = 224 */
  /* sum(i in N : Dtard[i] > 0) (Dfin[i] - Dtot[i]) <= 224; */
  /* Approche Bi-objectif Lexicographique : intgration de l'objectif du minimum de ressources, Z2 = 17.76 */
  /* sum (j in J) (Rmax[j]) <= 17.76; */
  
  /* Tout patient doit tre pris en charge  chaque tape de l'admission entre sa date au plus tt et sa date au plus tard  */
  forall (i in N, j in J) sum(t in P : t < Dtot[i]) Y[i][t][j] == 0;
  forall (i in N, j in J) sum(t in P : t > Dtard[i]) Y[i][t][j] == 0;
  forall (i in N : Dtard[i] > 0, j in J) sum(t in P) Y[i][t][j] == 1;

  /* Respect du flot des patients traversant les activits en squence, respect des prcdences entre activits */  
  forall (i in N : Dtard[i] > 0, j in L, k in J : Pred[j][k] > 0) Tach[i][j] - Dur[i][j] >= Tach[i][Pred[j][k]];
  forall (i in N : Dtard[i] > 0, j in J) sum (t in P) (Y[i][t][j] * t * Nuni) >= Tach[i][j];
  forall (i in N : Dtard[i] > 0) sum (t in P) (Y[i][t][1] * t * Nuni) == Tach[i][1];
    
  /* Respect de la capacit des ressources associes aux activits avec temps d'attente */
  forall(j in J) X[j][0]==0;
  
  forall(j in J, t in P) Cre[j][t] <= Cap[j][t];
  /* forall(j in J, t in P) Cre[j][t] <= Cap[j][t] * sum (i in N : Dtard[i] > 0) (Y[i][t][j] * TGV); */
  forall(j in J, t in P) X[j][t] <= Cre[j][t];
  forall(j in J, t in P) sum (i in N : Dtard[i] > 0) (Y[i][t][j] * Dur[i][j]) == (Cre[j][t] + X[j][t-1] - X[j][t]) * Nuni;
    
  /* calcul de la date de fin de l'ensemble des admissions */
  forall (i in N : Dtard[i] > 0, j in J, t in P) Cmax >= Y[i][t][j] * t;
  forall (j in J, t in P) Rmax[j] >= Cre[j][t];
  
  /* calcul des rsultats : dtail et valeurs des objectifs */
  forall (i in N : Dtard[i] > 0, t in P) Z[i][t] == Y[i][t][Mact] * t;
  forall (i in N : Dtard[i] > 0, j in J, t in P) Dfin[i] >= Y[i][t][j] * t;
  forall (i in N : Dtard[i] > 0) Dmin <= Dfin[i] - Dtot[i];
  forall (i in N : Dtard[i] > 0) Dmax >= Dfin[i] - Dtot[i];
  Dmoy == sum(i in N : Dtard[i] > 0) (Dfin[i] - Dtot[i]);
}